/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Mozilla Community QA Extension
 *
 * The Initial Developer of the Original Code is the Mozilla Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Zach Lipton <zach@zachlipton.com>
 *  Zach Linder <zakness@gmail.com>
 *  Ben Hsieh <ben.hsieh@gmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

@import url(chrome://global/skin/);

@namespace html "http://www.w3.org/1999/xhtml";

#qa-pass {
  background: #99ff99;
  display: none;
}

/*-------------------------------
               HTML
  -------------------------------*/

#qa_tabbox html|ul, #qa_tabbox html|ol {
  margin: 0;
  padding: 0;
}
#qa_tabbox html|ul li {
  list-style-type: square;
  margin-bottom: 0.4em;
}

#qa_tabbox html|a,
#qa_tabbox html|a:link,
#qa_tabbox html|a:visited {
  font-weight: bold;
  color: #0067ac;
}
#qa_tabbox html|a:focus,
#qa_tabbox html|a:hover,
#qa_tabbox html|a:active {
  color: #333;
}
/*remove bullets for everything but lists contained inside testcase data */
html|li {
  list-style-type: none !important;
}
#qa-tab-litmus html|ul html|li {
	list-style-type: disc !important;
}
#qa-tab-litmus html|ol html|li {
	list-style-type: decimal !important;
}

#qa-tab-litmus html|ul, #qa-tab-litmus html|ol {
	padding-left: 20px;
}



/*-------------------------------
               TABS
  -------------------------------*/
#qa_tabbox #qa_tabrow {
  height: 30px !important;
  padding-top: 5px !important;
}

#qa_tabbox .tab-icon-image {
  list-style-image: none;
  width: 0;
}

#qa_tabbox #qa-tabbar-qmo {
  list-style-image: url(qmo-16px.png);
  text-align: right;
  width: 24px;
}

#qa_tabbox #qa-tabbar-spacer {
  width: 93px;
}


#qa_tabbox tab, #qa_tabbox tabs spacer {
  border-bottom: 1px solid #ACA899 !important;
}

.tabs-left, .tabs-right {
  -moz-border-bottom-colors: #ACA899 !important;
}

#qa_tabbox tab[selected="true"] {
  border-bottom: 1px solid #E8E5D8 !important;
}

tabs {
  border-bottom: 1px solid #E8E5D8;
}

.tab-close-button {
    display: none !important;
}

.tabbrowser-tab.highlight > .tab-image-right {
	background: url("chrome://qa/skin/highlight-start.png") no-repeat !important;
}
.tabbrowser-tab.highlight > .tab-image-left {
	background: url("chrome://qa/skin/highlight-end.png") no-repeat !important;
}
.tabbrowser-tab.highlight > .tab-image-middle {
	background: url("chrome://qa/skin/highlight-mid.png") repeat-x !important;
}

.tabbrowser-tab.highlight:hover > .tab-image-right {
	background: url("chrome://qa/skin/highlight-hover-start.png") no-repeat !important;
}
.tabbrowser-tab.highlight:hover > .tab-image-left {
	background: url("chrome://qa/skin/highlight-hover-end.png") no-repeat !important;
}
.tabbrowser-tab.highlight:hover > .tab-image-middle {
	background: url("chrome://qa/skin/highlight-hover-mid.png") repeat-x !important;
}

/* don't show outline around the middle of selected tabs */
tab:focus > .tab-middle {
  outline: 1px none invert !important;
}

/*-------------------------------
               PANELS
  -------------------------------*/
#qa_tabbox tabpanel {
  line-height: 1.6;
  background-color: white;
}

#qa_tabbox tabpanels {
  background-color: white;
  border-top: 1px solid #ACA899;
  margin-top: 0px;
}

#qa_tabbox groupbox {
  background-color: white;
}
#qa_tabbox caption {
  font: bold 12px helvetica, arial, sans-serif;
  color: #ec891d;
  background-color: white;
  padding: 0;
}

#qa-qmo-help,
#qa-qmo-events,
#qa-qmo-forumposts,
.box-mheight {
}

#qa-qmo-latestbox,
#qa-qmo-latest {
  
}


/*-------------------------------
               NOTIFY
  -------------------------------*/

#qa-notify {
  right: 18px;
  bottom: 21px;
  position: fixed;
  border: 1px solid grey;
  background-color: white;
  background-image: url("logolight.png");
  background-repeat: no-repeat;
  background-position: center;
}

.qa-notify {
  height: 140px;
  width: 420px;
}

#qa-notify-box {
  padding: 0px 6px 6px 6px;
	display: table;
}

#qa-notify-header {
  font-size: 1.5em;
  font-weight: bold;
}


/*-------------------------------
               GENERAL
  -------------------------------*/

.list {
  padding-left: 10px;
}
