/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Joe Hewitt <hewitt@netscape.com>
 *   Håkan Waara <hwaara@chello.se>
 *   Jan Varga <varga@ku.sk>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* ===== folderPane.css =================================================
  == Styles for the Folder pane in the Messenger 3-pane window.
  ======================================================================= */

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

/* ::::: mail folder ::::: */

treechildren::-moz-tree-image(folderNameCol) {
  -moz-margin-end: 2px;
}

.icon-holder[type="folder"],
treechildren::-moz-tree-image(folderNameCol) {
  list-style-image: url("chrome://messenger/skin/icons/folder-closed.gif");
}

treechildren::-moz-tree-image(folderNameCol, newMessages-true) {
  list-style-image: url("chrome://messenger/skin/icons/folder-new.gif");
}

/* ..... News and Feeds ..... */

.icon-holder[type="folder"][ServerType="nntp"],
.icon-holder[type="folder"][ServerType="rss"],
treechildren::-moz-tree-image(folderNameCol, serverType-nntp),
treechildren::-moz-tree-image(folderNameCol, serverType-rss) {
  list-style-image: url("chrome://messenger/skin/icons/folder-newsgroup.gif");
}

.icon-holder[type="folder"][ServerType="nntp"][NewMessages="true"],
.icon-holder[type="folder"][ServerType="rss"][NewMessages="true"],
treechildren::-moz-tree-image(folderNameCol, serverType-nntp, newMessages-true),
treechildren::-moz-tree-image(folderNameCol, serverType-rss, newMessages-true) {
  list-style-image: url("chrome://messenger/skin/icons/folder-newsgroup-new.gif");
}

/* ..... Inbox ..... */

.icon-holder[type="folder"][SpecialFolder="Inbox"],
treechildren::-moz-tree-image(folderNameCol, specialFolder-Inbox) {
  list-style-image: url("chrome://messenger/skin/icons/folder-inbox.gif");
}

.icon-holder[type="folder"][SpecialFolder="Inbox"][NewMessages="true"],
treechildren::-moz-tree-image(folderNameCol, specialFolder-Inbox, newMessages-true) {
  list-style-image: url("chrome://messenger/skin/icons/folder-inbox-new.gif");
}

/* ..... Sent ..... */

.icon-holder[type="folder"][SpecialFolder="Sent"],
treechildren::-moz-tree-image(folderNameCol, specialFolder-Sent) {
  list-style-image: url("chrome://messenger/skin/icons/folder-sent.gif");
}

/* ..... Drafts ..... */

.icon-holder[type="folder"][SpecialFolder="Drafts"],
treechildren::-moz-tree-image(folderNameCol, specialFolder-Drafts) {
  list-style-image: url("chrome://messenger/skin/icons/folder-draft.gif");
}

/* ..... Templates ..... */

.icon-holder[type="folder"][SpecialFolder="Templates"],
treechildren::-moz-tree-image(folderNameCol, specialFolder-Templates) {
  list-style-image: url("chrome://messenger/skin/icons/folder-template.gif");
}

/* ..... Outbox ..... */

.icon-holder[type="folder"][SpecialFolder="Outbox"],
treechildren::-moz-tree-image(folderNameCol, specialFolder-Outbox) {
  list-style-image: url("chrome://messenger/skin/icons/folder-outbox.gif");
}

/* ..... Junk ..... */

.icon-holder[type="folder"][SpecialFolder="Junk"],
treechildren::-moz-tree-image(folderNameCol, specialFolder-Junk) {
  list-style-image: url("chrome://messenger/skin/icons/folder-junk.gif");
}

/* ..... Trash ..... */

.icon-holder[type="folder"][SpecialFolder="Trash"],
treechildren::-moz-tree-image(folderNameCol, specialFolder-Trash) {
  list-style-image: url("chrome://messenger/skin/icons/folder-trash.gif");
}

/* ..... Saved Searches ..... */

.icon-holder[type="folder"][SpecialFolder="Virtual"],
treechildren::-moz-tree-image(folderNameCol, specialFolder-Virtual) {
  list-style-image: url("chrome://messenger/skin/icons/folder-search.gif");
}

/* ...... Shared ..... */

.icon-holder[type="folder"][ImapShared="true"],
treechildren::-moz-tree-image(folderNameCol, imapShared-true) {
  list-style-image: url("chrome://messenger/skin/icons/folder-share.gif");
}

/* ..... Server Folders ..... */

.icon-holder[type="folder"][IsServer="true"],
treechildren::-moz-tree-image(folderNameCol, isServer-true) {
  list-style-image: url("chrome://messenger/skin/icons/server-mail.gif");
}

.icon-holder[type="folder"][BiffState="NewMail"][IsServer="true"],
treechildren::-moz-tree-image(folderNameCol, biffState-NewMail, isServer-true) {
  list-style-image: url("chrome://messenger/skin/icons/server-mail-new.gif");
}

.icon-holder[type="folder"][IsServer="true"][ServerType="pop3"][IsSecure="true"],
treechildren::-moz-tree-image(folderNameCol, isServer-true, serverType-pop3, isSecure-true) {
  list-style-image: url("chrome://messenger/skin/icons/server-remote-lock.gif");
}

.icon-holder[type="folder"][IsServer="true"][ServerType="imap"][IsSecure="true"],
treechildren::-moz-tree-image(folderNameCol, isServer-true, serverType-imap, isSecure-true) {
  list-style-image: url("chrome://messenger/skin/icons/server-remote-lock.gif");
}

.icon-holder[type="folder"][BiffState="NewMail"][IsServer="true"][IsSecure="true"],
treechildren::-moz-tree-image(folderNameCol, biffState-NewMail, isServer-true, isSecure-true) {
  list-style-image: url("chrome://messenger/skin/icons/server-remote-lock-new.gif");
}

.icon-holder[type="folder"][IsServer="true"][ServerType="none"]
treechildren::-moz-tree-image(folderNameCol, isServer-true, serverType-none) {
  list-style-image: url("chrome://messenger/skin/icons/server-local.gif");
}

.icon-holder[type="folder"][IsServer="true"][ServerType="nntp"],
treechildren::-moz-tree-image(folderNameCol, isServer-true, serverType-nntp) {
  list-style-image: url("chrome://messenger/skin/icons/server-news.gif");
}

.icon-holder[type="folder"][IsServer="true"][ServerType="nntp"][IsSecure="true"],
treechildren::-moz-tree-image(folderNameCol, isServer-true, serverType-nntp, isSecure-true) {
  list-style-image: url("chrome://messenger/skin/icons/server-news-lock.gif");
}

.icon-holder[type="folder"][IsServer="true"][ServerType="rss"],
treechildren::-moz-tree-image(folderNameCol, isServer-true, serverType-rss) {
  list-style-image: url("chrome://communicator/skin/icons/feedIcon16.png");
}

/* ::::: All Servers ::::: */

treechildren::-moz-tree-cell-text(closed, subfoldersHaveUnreadMessages-true),
treechildren::-moz-tree-cell-text(folderNameCol, isServer-true),
treechildren::-moz-tree-cell-text(hasUnreadMessages-true) {
  font-weight: bold;
}

treechildren::-moz-tree-cell-text(folderNameCol, noSelect-true) {
  color: gray;
  font-style: italic;
}

.tree-folder-checkbox {
  list-style-image: none;
}

treechildren::-moz-tree-image(syncCol) {
  list-style-image: url("chrome://global/skin/checkbox/cbox.gif");
}

treechildren::-moz-tree-image(syncCol, synchronize-true) {
  list-style-image: url("chrome://global/skin/checkbox/cbox-check.gif");
}

treechildren::-moz-tree-image(syncCol, isServer-true) {
  list-style-image: none;
}

#folderUnreadCol,
#folderTotalCol,
#folderSizeCol {
  text-align: right;
}

#folderNameCol [sortDirection="ascending"] {
  list-style-image: none;
}

#folderNameCol [sortDirection="descending"] {
  list-style-image: none;
}
