function TestCase(n, d, e, a) this.expect = e;
function reportCompare(expected, actual, description) {
    typeof actual
}
expect = 1;
var summary = 'Do not assert: top < ss->printer->script->depth';
var actual = 'No Crash';
var expect = 'No Crash';
test();
function test() {
    try {
        p = [1].some(function (y) test()) ? 4 : 0x0041
    } catch (ex) {}
    reportCompare(expect, actual, summary)
}
test();
TestCase();
test()
