/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998-1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* ===== messenger.css ==================================================
  == Content specific styles for Messenger.
  ======================================================================= */

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

/* ::::: mail xbl bindings ::::: */

.autocomplete-richlistitem[type="websearch"] {
  -moz-binding: url("chrome://messenger/content/webSearch.xml#websearch");
}

/* attachment list styles */

attachmentlist {
  -moz-appearance: listbox;
  -moz-user-focus: normal;
}

attachmentlist[orient="horizontal"] {
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#attachmentlist-horizontal");
}

attachmentlist[orient="vertical"] {
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#attachmentlist-vertical");
}

.attachmentlist-wrapper {
  display: block;
  margin: 0;
  padding: 0;
  line-height: 0;
}

attachmentitem {
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#attachmentitem");
}

.attachmentcell-content {
  -moz-box-orient: horizontal;
}

.attachmentcell-text {
  -moz-box-orient: horizontal;
}

attachmentlist[view="large"] .attachmentcell-content {
  -moz-box-orient: horizontal;
}

attachmentlist[view="large"] .attachmentcell-text {
  -moz-box-align: start;
  -moz-box-orient: vertical;
}

attachmentlist[view="tile"] .attachmentcell-content {
  -moz-box-align: center;
  -moz-box-orient: vertical;
}

attachmentlist[view="tile"] .attachmentcell-text {
  -moz-box-align: center;
  -moz-box-orient: vertical;
}

attachmentlist[orient="horizontal"] > attachmentitem {
  min-width: 10em;
}

attachmentlist[orient="horizontal"][view="tile"] > attachmentitem {
  min-width: 5em;
  max-width: 10em;
}

/* message header widgets */

mail-messageid {
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#mail-messageid");
}

mail-messageids-headerfield {
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#mail-messageids-headerfield");
}

mail-emailaddress {
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#mail-emailaddress");
}

.emailDisplayButton {
  -moz-user-focus: normal;
}

.emailStar {
  -moz-user-focus: normal;
}

.emailPopup {
  -moz-user-focus: normal;
}

mail-emailheaderfield {
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#mail-emailheaderfield");
}

mail-toggle-headerfield {
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#mail-toggle-headerfield");
}

mail-multi-emailHeaderField {
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#mail-multi-emailHeaderField");
}

mail-newsgroup {
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#mail-newsgroup");
}

mail-newsgroups-headerfield {
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#mail-newsgroups-headerfield");
}

mail-headerfield {
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#mail-headerfield");
}

mail-urlfield {
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#mail-urlfield");
}

mail-tagfield {
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#mail-headerfield-tags");
}

mail-tagfield[collapsed="true"] {
  display: none;
}

searchattribute {
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#searchattribute");
}

searchoperator {
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#searchoperator");
}

searchvalue {
  display: -moz-deck;
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#searchvalue");
}

searchterm {
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#searchterm");
}

.ruleaction {
  -moz-binding: url("chrome://messenger/content/searchWidgets.xml#ruleaction"); 
}

.ruleactiontype {
  -moz-binding: url("chrome://messenger/content/searchWidgets.xml#ruleactiontype-menulist"); 
}

.ruleactiontarget[type] {
  -moz-binding: url("chrome://messenger/content/searchWidgets.xml#ruleactiontarget-base");
}

.ruleactiontarget[type="movemessage"], .ruleactiontarget[type="copymessage"] { 
  -moz-binding: url("chrome://messenger/content/searchWidgets.xml#ruleactiontarget-folder");
}

.ruleactiontarget[type="addtagtomessage"] {
  -moz-binding: url("chrome://messenger/content/searchWidgets.xml#ruleactiontarget-tag");
}

.ruleactiontarget[type="setpriorityto"] {
  -moz-binding: url("chrome://messenger/content/searchWidgets.xml#ruleactiontarget-priority");
}

.ruleactiontarget[type="setjunkscore"] {
  -moz-binding: url("chrome://messenger/content/searchWidgets.xml#ruleactiontarget-junkscore");
}

.ruleactiontarget[type="forwardmessage"] {
  -moz-binding: url("chrome://messenger/content/searchWidgets.xml#ruleactiontarget-forwardto");
}

.ruleactiontarget[type="replytomessage"] {
  -moz-binding: url("chrome://messenger/content/searchWidgets.xml#ruleactiontarget-replyto");
}

dummy.usesMailWidgets {
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#dummy");
}

#searchInput {
  -moz-binding: url("chrome://messenger/content/search.xml#glodaSearch");
}

.quick-search-button {
  -moz-binding: url("chrome://messenger/content/search.xml#searchBarDropMarker");
  cursor: default;
  -moz-user-focus: none;
}

.quick-search-clearbutton{
  cursor: default;
  -moz-user-focus: none;
}

.folderTargetPopup {
  display: -moz-popup;
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#folderTargetPopup");
}

.searchPopup {
  display: -moz-popup;
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#searchpopup");
}

.folderSummaryPopup
{
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#folderSummary-popup");
}

folderSummary
{
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#folderSummary");
}

folderSummaryMessage
{
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#folderSummary-message");
}

folderSummaryLocation
{
  -moz-binding: url("chrome://messenger/content/mailWidgets.xml#folderSummary-location");
}

.foldersTree
{
  margin: 0px;
  border: none;
  -moz-user-focus: ignore;
}

.foldersTreeChildren
{
  -moz-binding: url("chrome://global/content/bindings/tree.xml#tree-base");
}

.foldersTreeChildren::-moz-tree-twisty
{
  width: 0px;
}

tabmail {
  -moz-binding: url("chrome://messenger/content/tabmail.xml#tabmail");
}

menupopup[type="folder"] {
  -moz-binding: url("chrome://messenger/content/folderWidgets.xml#folder-menupopup");
}

.addrbooksPopup {
  -moz-binding: url("chrome://messenger/content/addressbook/addrbookWidgets.xml#addrbooks-menupopup");
}

#threadCols > treecolpicker {
  -moz-binding: url("chrome://messenger/content/threadPaneColumnPicker.xml#threadPaneColumnPicker");
}

/**
 * Toolboxes default to overflow: visible, and we need to override that to
 * prevent the toolbox from causing its containing box (the entire content
 * area) to become wider than the viewport, which has the bad side-effect of
 * causing the _other_ children (i.e. tabmail, which contains important widgetry
 * like the message scrollbar) to be rendered out of view.
 *
 * As it turns out, we only need to override the mail-bar3 (at least of
 * the toolbars we create), and we only need to override the overflow-x to
 * fix this problem, so let's do that.  This does open us up to extension
 * toolbars causing the same problem, but we should be able to get the
 * authors to change their extensions to prevent this problem.
 */
#mail-bar3 {
  overflow-x: hidden;
}

/* Lightweight themes support */

#messengerWindow:-moz-lwtheme {
  background-repeat: no-repeat;
  background-position: top right;
}

#status-bar[lwthemefooter="true"] {
  background-repeat: no-repeat;
  background-position: bottom left;
}
