/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Thunderbird Default Client Dialog
 *
 * The Initial Developer of the Original Code is
 *   Scott MacGregor <mscott@mozilla.org>
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Mark Banner <bugzilla@standard8.demon.co.uk>
 *   Frank Wein <mcsmurf@mcsmurf.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// this dialog can only be opened if we have a shell service
const nsIShellService = Components.interfaces.nsIShellService;
const nsIPrefBranch = Components.interfaces.nsIPrefBranch;

function onLoad()
{
  var shellSvc = Components.classes["@mozilla.org/suite/shell-service;1"]
                           .getService(nsIShellService);
  var defaultList = document.getElementById("defaultList");
  var appTypes = shellSvc.shouldBeDefaultClientFor;
  /* Iterate through the list of possible default client types and check for
     each list item if we want to be the default for that type using the AND
     conjunction */
  for (var i = 0; i < defaultList.getRowCount(); i++) {
    var currentItem = defaultList.getItemAtIndex(i);
    if (nsIShellService[currentItem.value] & appTypes)
      currentItem.checked = true;

    if (shellSvc.isDefaultClient(false, nsIShellService[currentItem.value]))
      currentItem.disabled = true;
  }
}

function onAccept()
{
  // for each checked item, if we aren't already the default, make us the default.
  var shellSvc = Components.classes["@mozilla.org/suite/shell-service;1"]
                           .getService(nsIShellService);
  var pref = Components.classes["@mozilla.org/preferences-service;1"]
                       .getService(nsIPrefBranch);
  var appTypes = 0;
  var appTypesCheck = 0;
  var defaultList = document.getElementById("defaultList");

  for (var i = 0; i < defaultList.getRowCount(); i++) {
    var currentItem = defaultList.getItemAtIndex(i);
    var currentAppType = nsIShellService[currentItem.value];
    
    if (currentItem.checked) {
      appTypesCheck |= currentAppType;
      
      if (!currentItem.disabled)
        appTypes |= currentAppType;
    }
  }
 
  if (appTypes)
    shellSvc.setDefaultClient(false, true, appTypes);

  // Update the pref for which app types we should check if we are the default app
  shellSvc.shouldBeDefaultClientFor = appTypesCheck;

  shellSvc.shouldCheckDefaultClient = document.getElementById('checkOnStartup').checked;
}
