/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Oracle Corporation code.
 *
 * The Initial Developer of the Original Code is Oracle Corporation
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Stuart Parmenter <pavlov@pavlov.net>
 *   Joey Minta <jminta@gmail.com>
 *   Fred Jendrzejewski <fred.jen@web.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsISupports.idl"

interface nsIURI;
interface calIIcalProperty;
interface calIItemBase;

[scriptable,uuid(327a73e3-ba20-4dfc-92b9-80446b7d0d1b)]
interface calIAttachment : nsISupports
{
    /**
     * The hash id is used to identify this attachment and compare it to others.
     */
    readonly attribute AUTF8String hashId;

    /**
     * An nsIURI object that points to the file (local or remote)
     */
    attribute nsIURI uri;

    /**
     * Raw attachment data, in case its not an uri
     */
    attribute AUTF8String rawData;

    /**
     * The type of file that this attachment refers to
     */
    attribute AString formatType;

    /**
     * The encoding the (local) file should be encoded with.
     */
    attribute AUTF8String encoding;

    /**
     * The calIIcalProperty corresponding to this object.  Can be used for
     * serializing/unserializing from ics files.
     */
    attribute calIIcalProperty icalProperty;

    /**
     * For accessing additional parameters, such as x-params.
     */
    AUTF8String getParameter(in AString name);
    void setParameter(in AString name, in AUTF8String value);
    void deleteParameter(in AString name);

    /**
     * Clone this calIAttachment instance into a new object.
     */
    calIAttachment clone();
};
