/*
# -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 4 -*-
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is the Thunderbird Preferences System.
#
# The Initial Developer of the Original Code is
# Ben Goodger.
# Portions created by the Initial Developer are Copyright (C) 2005
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   Scott MacGregor <mscott@mozilla.org
#   Kevin Gerich <webmail@kmgerich.com>
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****
*/

.windowDialog {
  padding: 12px;
  font: -moz-dialog;
}

#permissionsText {
  font: small-caption;
  font-weight: normal;
  line-height: 1.3em;
  margin-bottom: 4px !important;
}

menulist > menupopup > menuitem {
  font: -moz-pull-down-menu;
}
 
prefwindow:not([type="child"]) {
  padding: 0 !important;
}

prefpane {
  padding: 12px;
}

.paneSelector {
  list-style-image: url("chrome://messenger/skin/preferences/mail-options.png");
}

radio[pane=paneGeneral] {
  -moz-image-region: rect(0px, 32px, 32px, 0px)
}

radio[pane=paneDisplay] {
  -moz-image-region: rect(0px, 64px, 32px, 32px)
}

radio[pane=paneCompose] {
  -moz-image-region: rect(0px, 96px, 32px, 64px)
}

radio[pane=paneSecurity] {
  -moz-image-region: rect(0px 128px 32px 96px);
}

radio[pane=paneApplications] {
  -moz-image-region: rect(0px 160px 32px 128px);
}

radio[pane=paneAdvanced] {
  -moz-image-region: rect(0px, 256px, 32px, 224px)
}

/* Attachments Pane */
#MailPreferences[animated="true"] #handlersView {
  height: 25em;
}

#MailPreferences[animated="false"] #handlersView {
  -moz-box-flex: 1;
}

#browserStartupHomepage {
  padding-top: 2px;
  padding-bottom: 3px;
  -moz-padding-start: 4px;
  -moz-padding-end: 2px;
}

prefpane .groupbox-body {
  -moz-appearance: none;
  padding: 8px 4px 4px;
}

#paneTabs > groupbox {
  margin: 0;
}

#paneTabs > vbox {
  margin: 8px;
}

prefpane .groupbox-title {
  background: url("chrome://global/skin/50pct_transparent_grey.png") repeat-x bottom left;
  margin-bottom: 4px;
}

caption {
  border-bottom: 1px solid #ECECEC;
  padding-top: 4px !important;
  padding-bottom: 0px !important;
  -moz-padding-start: 0px !important;
  -moz-padding-end: 4px !important;
}

#paneContent {
  padding-top: 8px;
}

#paneContent row {
  padding: 0 4px;
}

#advancedPrefs {
  margin: 0 8px;
}

#securityPrefs {
  padding: 0 4px;
}

tabpanels {
  padding: 18px 10px 10px;
  margin: 0 6px 6px;
}

/* override the color style for tag items when they
   have selection */
#tagList listitem[selected="true"] {
  color: -moz-DialogText !important;
}

#tagList:focus > listitem[selected="true"] {
  color: HighlightText !important;
}
