/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Joe Hewitt <hewitt@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* ==== toolbar.css =====================================================
   == Styles for toolbars that are used throughout the Communicator suite.
   ====================================================================== */

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

/* ::::: toolbar-primary ::::: */

.toolbar-primary {
  -moz-binding: url("chrome://communicator/skin/toolbar/toolbarBindings.xml#toolbar-primary");
  background-color: #B1BDC9;
}

.toolbar-primary-holder {
  background: url("chrome://communicator/skin/toolbar/prtb-bg-line.gif") repeat-x top;
  border-top: none;
  border-right: 1px solid #95A0AD;
  border-bottom: 1px solid #95A0AD;
  border-left: none;
}

.toolbar-primary-holder[mode="icons"],
.toolbar-primary-holder[mode="text"],
.toolbar-primary-holder[iconsize="small"],
.toolbar-primary-holder[labelalign="end"] {
  background-image: url("chrome://communicator/skin/toolbar/prtb-bg-noline.gif");
}

.toolbar-primary-icon {
  width: 77px;
  height: 50px;
  background-repeat: no-repeat;
  background-position: top;
}

.toolbar-primary-icon[mode="icons"],
.toolbar-primary-icon[mode="text"],
.toolbar-primary-icon[iconsize="small"],
.toolbar-primary-icon[labelalign="end"] {
  display: none;
}

toolbar[mode="text"] toolbarbutton[type="menu-button"],
toolbar[labelalign="end"]:not([mode="icons"]) toolbarbutton[type="menu-button"] {
  -moz-binding: url("chrome://global/content/bindings/toolbarbutton.xml#menu-button");
}

toolbar[mode="text"] .toolbarbutton-1 .toolbarbutton-text,
toolbar[labelalign="end"] .toolbarbutton-1 .toolbarbutton-text {
  min-width: 0px;
}

/* ::::: toolbargrippy ::::: */
 
.toolbar-primary-grippy { 
  -moz-binding: url("chrome://communicator/skin/toolbar/toolbarBindings.xml#toolbargrippy-primary");
  border: none;
  background: url("chrome://communicator/skin/toolbar/prtb-grip-mid.gif") repeat-y;
  list-style-image: url("chrome://communicator/skin/toolbar/prtb-grip-btm.gif");
} 

.toolbar-primary-grippy > .toolbargrippy-texture {
  width: 13px;
  height: 0px;
  list-style-image: url("chrome://communicator/skin/toolbar/prtb-grip-top.gif");
}

.toolbar-primary-grippy > .toolbargrippy-arrow {
  margin: 0px;
  width: 13px;
  height: 10px;
  list-style-image: inherit;
}

.toolbar-primary-grippy:hover:active {
  background-image: url("chrome://communicator/skin/toolbar/prtb-grip-mid-act.gif");
  list-style-image: url("chrome://communicator/skin/toolbar/prtb-grip-btm-act.gif");
}

.toolbar-primary-grippy:hover:active > .toolbargrippy-texture {
  list-style-image: url("chrome://communicator/skin/toolbar/prtb-grip-top-act.gif");
}

/* ::::: toolbar-primary separator ::::: */

.toolbar-primary > toolbarseparator,
.toolbarseparator-primary {
  margin: 0px;
  padding: 0px;
  border: none;
  width: 18px;
  background: none;
}

.toolbar-primary > toolbarpaletteitem > toolbarseparator {
  margin: 0px 8px;
}

/* ::::: solid background status bar panel ::::: */

.statusbarpanel-backgroundbox {
  -moz-box-align: stretch;
  padding: 0px;
}

.statusbarpanel-backgroundbox > .statusbarpanel-contentbox {
  padding: 0px 4px;
  -moz-box-align: center;
}
